@@ -34,6 +34,8 @@
 #include <unistd.h>
 #include <time.h>
 
+#include "authentication.c"
+
 static const gchar *
 type_to_string (GnomeVFSFileType type)
 {
@@ -59,6 +61,73 @@ type_to_string (GnomeVFSFileType type)
 	}
 }
 
+static void
+show_acl (GnomeVFSACL *acl)
+{
+	GList *ace_list;
+	GList *iter;
+	
+	printf ("ACLs              :\n");
+	
+	ace_list = gnome_vfs_acl_get_ace_list (acl);
+	
+	for (iter = ace_list; iter; iter = iter->next) {
+		GnomeVFSACLKind  kind;
+		GnomeVFSACE     *ace;
+		GnomeVFSACLPerm *perms;
+		gboolean         negative;
+		gboolean         inherit;
+		char            *id;
+		const char      *kind_str;
+		
+		ace = GNOME_VFS_ACE (iter->data);
+		
+		kind = GNOME_VFS_ACL_KIND_NULL;
+		inherit = FALSE;
+		negative = FALSE;
+		id = "";
+		perms = NULL;
+		
+		g_object_get (ace,
+		              "kind", &kind, 
+		              "id", &id,
+		              "permissions", &perms,
+		              "negative", &negative,
+		              "inherit", &inherit,
+		              NULL);
+	
+		kind_str = gnome_vfs_acl_kind_to_string (kind);
+		
+		if (kind == GNOME_VFS_ACL_KIND_NULL) {
+			continue;	
+		}
+		
+		printf ("                  : %s:%s:", kind_str, id);
+		
+		if (perms != NULL) {
+			GnomeVFSACLPerm *piter;
+			const char *pstr;
+			
+			for (piter = perms; *piter; piter++) {
+				pstr = gnome_vfs_acl_perm_to_string (*piter);
+				printf ("%s ", pstr);
+			}	
+		}
+		
+		if (inherit || negative) {
+			if (inherit && negative) {
+				printf (" (negative, inherit)");
+			} else {
+				printf (" (%s)", negative ? "negative" : "inherit");	
+			}
+		}
+		
+		printf ("\n");
+	}
+	
+	gnome_vfs_acl_free_ace_list (ace_list);
+}
+
 static void
 show_file_info (GnomeVFSFileInfo *info, const char *uri)
 {
@@ -85,6 +154,9 @@ show_file_info (GnomeVFSFileInfo *info, const char *uri)
 				gnome_vfs_mime_application_get_desktop_id (app));
 		}
 	}
+	
+	if (info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_SELINUX_CONTEXT)
+		printf ("SELinux Context   : %s\n", info->selinux_context);
 
 	if(info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_SIZE)
 		printf ("Size              : %" GNOME_VFS_SIZE_FORMAT_STR "\n",
@@ -138,6 +210,10 @@ show_file_info (GnomeVFSFileInfo *info, const char *uri)
 		printf ("Executable        : %s\n",
 				(info->permissions&GNOME_VFS_PERM_ACCESS_EXECUTABLE?"YES":"NO"));
 	}
+	
+	if(info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_ACL) {
+		show_acl (info->acl);	
+	}
 
 #undef FLAG_STRING
 }
@@ -179,12 +255,16 @@ main (int argc, char **argv)
 		return 1;
 	}
 
+	command_line_authentication_init ();
+
 	text_uri = gnome_vfs_make_uri_from_shell_arg (argv[1]);
 	
 	info = gnome_vfs_file_info_new ();
 	
 	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE | 
-		  GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+		  GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS |
+		  GNOME_VFS_FILE_INFO_GET_ACL |
+		  GNOME_VFS_FILE_INFO_GET_SELINUX_CONTEXT;
 
 	if (slow_mime) {
 		options |= GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE;
